from ostrich.stats_provider import StatsProvider
from ostrich.stats_collection import StatsCollection
from ostrich.timing import TimingStat

class Stats(StatsProvider):
    def __init__(self):
        self.gauges = {}
        self.collection = StatsCollection()
        self.forked_collections = []
    
    def add_timing(self, name, timing):
        map(lambda c: c.add_timing(name, timing), self.forked_collections)
        return self.collection.add_timing(name, timing)
    
    def incr(self, name, count=1):
        map(lambda c: c.incr(name, count), self.forked_collections)
        return self.collection.incr(name, count)
    
    def get_counter_stats(self, reset=False):
        return self.collection.get_counter_stats(reset)
    
    def get_timing_stats(self, reset=False):
        return self.collection.get_timing_stats(reset)
    
    def get_gauge_stats(self):
        return dict(map(lambda (k, gauge): (k, gauge()), self.gauges.items()))
    
    def get_timing(self, name):
        # make sure any new timings get added to forked collections
        map(lambda c: c.get_timing(name), self.forked_collections)
        return self.collection.get_timing(name)
    
    def get_counter(self, name):
        # make sure any new counters get added to forked collections
        map(lambda c: c.get_counter(name), self.forked_collections)
        return self.collection.get_counter(name)
    
    def stats(self, reset=False):
        d = self.collection.stats(reset) 
        d.update(gauges=self.get_gauge_stats())
        return d
    
    def clear_all(self):
        map(lambda c: c.clear_all(), self.forked_collections)
        del self.forked_collections[:]
        self.collection.clear_all()
        self.gauges.clear()
    
    def make_gauge(self, name, func):
        self.gauges[name] = func
    
    def fork(self):
        collection = StatsCollection()
        self.forked_collections.append(collection)
        return collection

_stats = Stats()
add_timing = _stats.add_timing
incr = _stats.incr
get_counter_stats = _stats.get_counter_stats
get_timing_stats = _stats.get_timing_stats
get_gauge_stats = _stats.get_gauge_stats
get_timing = _stats.get_timing
get_counter = _stats.get_counter
clear_all = _stats.clear_all
make_gauge = _stats.make_gauge
stats = _stats.stats
time = _stats.time
time_ns = _stats.time_ns
fork = _stats.fork

def json_encoder(o):
    if isinstance(o, TimingStat):
        return o.to_dict(raw_histogram=True)
    else:
        raise TypeError(repr(o) + " is not JSON serializable")

def gauge(name):
    def _decorator(func):
        make_gauge(name, func)
        return func
    return _decorator


import json

from twisted.web import static
from twisted.web.resource import Resource

from ostrich import stats
from ostrich.time_series_collector import TimeSeriesCollector

class StatsResource(Resource):
    def render_GET(self, request):
        reset = int(request.args.get('reset', [0])[0])
        return json.dumps(stats.stats(reset=reset), default=stats.json_encoder)

class StatsTimeSeriesResource(StatsResource):
    def __init__(self):
        Resource.__init__(self)
        self.collector = TimeSeriesCollector()
        self.collector.start_twisted()
        
        self.putChild('graph', static.Data(GRAPH_HTML.strip(), "text/html"))
        self.putChild('graph_data', TimeSeriesDataResource(self.collector))
    
    # def render_GET(self, request):
    #     reset = int(request.args.get('reset', [0])[0])
    #     return json.dumps(stats.stats(reset=reset), default=stats.json_encoder)

class TimeSeriesDataResource(Resource):
    isLeaf = True
    
    def __init__(self, collector):
        self.collector = collector
    
    def render_GET(self, request):
        if len(request.postpath) == 0:
            return json.dumps({'keys': self.collector.keys()}) + "\n"
        else:
            return json.dumps(self.collector.get(request.postpath[0])) + "\n"


GRAPH_HTML = """
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
<html>
<head>
  <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
  <script type="text/javascript" src="http://www.google.com/jsapi"></script>
  <script type="text/javascript" src="http://danvk.org/dygraphs/dygraph-combined.js"></script>
  <script type="text/javascript" src="http://ajax.googleapis.com/ajax/libs/jquery/1.4.1/jquery.min.js"></script>
  <script type="text/javascript">
    if (document.location.search.length > 0) {
      google.load('visualization', '1');
      google.setOnLoadCallback(drawChart);
    } else {
      $.getJSON("graph_data", function(datadump) {
        var keys = datadump["keys"].sort();
        for (i in keys) {
          $("#contents").append('<a href="graph?' + keys[i] + '">' + keys[i] + '</a><br/>');
        }
        $("#graph-container").css("display", "none");
      });
    }
    
    function roundTo(number, digits) {
      return Math.round(number * Math.pow(10, digits)) / Math.pow(10, digits);
    }

    function drawChart() {
      var key = document.location.search.substr(1);
      $.getJSON("graph_data/" + key, function(datadump) {
        var rawData = datadump[key];
        var data = new google.visualization.DataTable();
        data.addColumn('datetime', 'Time');
        for (i = 0; i < rawData[0].length - 1; i++) {
          data.addColumn('number', 'Data' + i);
        }
        data.addRows(rawData.map(function(row) { return [ new Date(row[0] * 1000) ].concat(row.slice(1)); }));

        new Dygraph.GVizChart(document.getElementById('chart')).draw(data, {
          includeZero: true,
          fillGraph: true,
          labelsKMG2: true,
          xAxisLabelFormatter: function(date, granularity) { return date.strftime("%H:%M"); },
          labelsDivStyles: { display: "none" },
          highlightCallback: function(e, x, pts) {
            var xloc = Math.floor(pts[pts.length - 1].canvasx) + 15;
            var yloc = Math.floor(pts[pts.length - 1].canvasy) + 15;
            var label = pts.map(function(p) { return roundTo(p.yval, 3); }).join(", ");
            $('#chart_label').html(label);
            $('#chart_label').css({ display: "block", left: xloc + "px", top: yloc + "px" });
          },
          unhighlightCallback: function(e) {
            $('#chart_label').css({ display: "none" });
          },
        });
      });
    }
  </script>
</head>
<body>

<div id="graph-container">
<div id="chart" style="width: 640px; height: 320px;"></div>
<span id="chart_label" style="position: absolute; font-size: 10pt;"></span>
</div>
<div id="contents"></div>

</body>
</html>

"""


import time

from decorator import decorator

class StatsProvider(object):
    def time(self, name):
        """Returns a Timer that logs the duration, in milliseconds, with the given name."""
        return Timer(self, name)
    
    def time_ns(self, name):
        """Returns a Timer that logs the duration, in nanoseconds, with the given name.
        
        When using nanoseconds, be sure to encode your field with that fact. Consider
        using the suffix `_ns` in your field.
        """
        return Timer(self, name)
    
    def stats(self, reset=False):
        return dict(counters=self.get_counter_stats(reset), timings=self.get_timing_stats(reset))

    ##
    ##
    ##

    def add_timing(self, name, timing):
        return 0
    
    def incr(self, name, count=1):
        return count
    
    def get_counter_stats(self, reset=False):
        return {}
    
    def get_timing_stats(self, reset=False):
        return {}
    
    def clear_all(self):
        pass

class Timer(object):
    def __init__(self, provider, key, nano=False):
        self.provider = provider
        self.key = key
        self.nano = nano

    def __enter__(self):
        self.start = time.time()

    def __exit__(self, type, value, tb):
        self.end = time.time()
        if self.nano:
            self.provider.add_timing(self.key, int(self.duration() * 1000000000))
        else:
            self.provider.add_timing(self.key, int(self.duration() * 1000))

    def duration(self):
        return self.end - self.start

    def __call__(self, f):
        def _decorator(func, *args, **kwargs):
            with self:
                return func(*args, **kwargs)
        return decorator(_decorator, f)


from ostrich.stats_provider import StatsProvider
from ostrich.timing import Timing

class StatsCollection(StatsProvider):
    def __init__(self):
        self.counters = {}
        self.timings = {}
    
    def add_timing(self, name, duration):
        self.get_timing(name).add(duration)
    
    def incr(self, name, count=1):
        self.get_counter(name).incr(count)
    
    def get_counter_stats(self, reset=False):
        return dict(map(lambda (k, counter): (k, counter.get(reset)), self.counters.items()))
    
    def get_timing_stats(self, reset=False):
        return dict(map(lambda (k, timing): (k, timing.get(reset)), self.timings.items()))
    
    def get_counter(self, name):
        counter = self.counters.get(name, None)
        if counter is None:
            counter = Counter()
            self.counters[name] = counter
        return counter
            
    
    def get_timing(self, name):
        timing = self.timings.get(name, None)
        if timing is None:
            timing = Timing()
            self.timings[name] = timing
        return timing
    
    def clear_all(self):
        self.counters.clear()
        self.timings.clear()

class Counter(object):
    def __init__(self):
        self.value = 0;
    
    def incr(self, n=1):
        self.value += n
        return self.value
    
    def __call__(self):
        return self.value
    
    def reset(self):
        self.value = 0
        return self.value
    
    def get(self, reset=False):
        try:
            return self.value
        finally:
            if reset:
                self.reset()
    
    def __eq__(self, other):
        return self.value == other.value


import sys

class Histogram(object):
    BUCKET_OFFSETS = [1, 2, 3, 4, 5, 7, 9, 11, 14, 18, 24, 31, 40, 52, 67, 87, 113, 147, 191, 248,
          322, 418, 543, 706, 918, 1193, 1551, 2016, 2620, 3406, 4428, 5757, 7483,
          9728, 12647, 16441, 21373, 27784, 36119]
    
    @classmethod
    def _binary_search(cls, array, key, low, high):
        if low > high:
            return low
        else:
            mid = (low + high + 1) >> 1
            mid_val = array[mid]
            if mid_val < key:
                return cls._binary_search(array, key, mid + 1, high)
            elif mid_val > key:
                return cls._binary_search(array, key, low, mid - 1)
            else:
                # exactly equal to this bucket's value. but the value is an exclusive max, so bump it up.
                return mid + 1
    
    @classmethod
    def binary_search(cls, key):
        return cls._binary_search(cls.BUCKET_OFFSETS, key, 0, len(cls.BUCKET_OFFSETS) - 1)
    
    def __init__(self, *values):
        self.num_buckets = len(self.BUCKET_OFFSETS) + 1
        self.buckets = [0] * self.num_buckets
        self.total = 0
        if values:
            for val in values:
                self.add(val)
    
    def add(self, n):
        index = self.binary_search(n)
        self.buckets[index] += 1
        self.total += 1
    
    def clear(self):
        for i in range(self.num_buckets):
            self.buckets[i] = 0
        self.total = 0

    def get_percentile(self, percentile):
        sum = 0
        index = 0
        while sum < percentile * self.total:
            sum += self.buckets[index]
            index += 1
        
        if index == 0:
            return 0
        elif index - 1 >= len(self.BUCKET_OFFSETS):
            return sys.maxint
        else:
            return self.BUCKET_OFFSETS[index - 1] - 1
    
    def merge(self, other):
        for i in range(self.num_buckets):
            self.buckets[i] += other.buckets[i]
        self.total += other.total
    
    def clone(self):
        new = Histogram()
        new.merge(self)
        return new


import sys
import math

from ostrich.histogram import Histogram

class Timing(object):
    def __init__(self):
        self.max = 0
        self.min = sys.maxint
        self.count = 0
        self.histogram = Histogram()
        self.mean = 0.0
        self.partial_variance = 0.0
    
    def clear(self):
        self.max = 0
        self.min = sys.maxint
        self.count = 0
        self.histogram.clear()
    
    def add(self, n):
        if isinstance(n, TimingStat):
            return self.add_timing_stat(n)
        else:
            return self.add_duration(n)
    
    def add_duration(self, n):
        if n > -1:
            self.max = max(self.max, n)
            self.min = min(self.min, n)
            self.count += 1
            self.histogram.add(n)
            if self.count == 1:
                self.mean = float(n)
                self.partial_variance = 0.0
            else:
                new_mean = self.mean + (n - self.mean) / self.count
                self.partial_variance += (n - self.mean) * (n - new_mean)
                self.mean = new_mean
        else:
            # TODO: warning?
            pass
        return self.count
    
    def add_timing_stat(self, timing_stat):
        if timing_stat.count > 0:
            # (comment from Scala ostrich) these equations end up using the sum again, and may be lossy. i couldn't find or think of
            # a better way.
            new_mean = (self.mean * self.count + timing_stat.mean * timing_stat.count) / (self.count + timing_stat.count)
            self.partial_variance = self.partial_variance + timing_stat.partial_variance + \
                                    (self.mean - new_mean) * self.mean * self.count + \
                                    (timing_stat.mean - new_mean) * timing_stat.mean * timing_stat.count
            self.mean = new_mean
            self.count += timing_stat.count
            self.max = max(self.max, timing_stat.max)
            self.min = min(self.min, timing_stat.min)
            if timing_stat.histogram is not None:
                self.histogram.merge(timing_stat.histogram)
    
    def get(self, reset=False):
        try:
            return TimingStat(self.count, self.max, self.min, self.mean, self.partial_variance, self.histogram.clone())
        finally:
            if reset:
                self.clear()

class TimingStat(object):
    """A pre-calculated timing. If you have timing stats from an external source but
    still want to report them via the Stats interface, use this.
       
    Partial variance is `(count - 1)(s^2)`, or `sum(x^2) - sum(x) * mean`.
    """
    def __init__(self, count, max, min, mean=0.0, partial_variance=0.0, histogram=None):
        self.count = count
        self.min = min if count > 0 else 0
        self.max = max if count > 0 else 0
        self.average = int(mean) if count > 0 else 0
        self.mean = mean if count > 0 else 0.0
        self.partial_variance = partial_variance if count > 1 else 0.0
        self.variance = (partial_variance / (count - 1)) if count > 1 else 0.0
        self.std_dev = round(math.sqrt(self.variance))
        self.histogram = histogram
    
    def __eq__(self, other):
        return self.count == other.count and self.max == other.max and \
               self.min == other.min and self.average == other.average and self.variance == other.variance
    
    def to_dict_no_histogram(self):
        return dict(count=self.count, maximum=self.max, minimum=self.min,
                    average=self.average, standard_deviation=long(self.std_dev))
    
    def to_dict(self, percentiles=True, raw_histogram=False):
        d = self.to_dict_no_histogram()
        if self.histogram:
            h = self.histogram
            d.update(p25=h.get_percentile(0.25),
                     p50=h.get_percentile(0.50),
                     p75=h.get_percentile(0.75),
                     p90=h.get_percentile(0.90),
                     p99=h.get_percentile(0.99),
                     p999=h.get_percentile(0.999),
                     p9999=h.get_percentile(0.9999))
            if raw_histogram:
                # strip off all the zeros at the end of the histogram
                histogram = list(h.buckets)
                while histogram and histogram[-1] == 0:
                    histogram = histogram[:-1]
                d.update(histogram=histogram)
        return d
    
    def __str__(self):
        return "(" + ", ".join(["%s=%d" % (k, v) for k, v in sorted(self.to_dict().items())]) + ")"


from __future__ import absolute_import

import time

from ostrich import stats

class TimeSeries(object):
    def __init__(self, size, empty=0):
        self.data = [empty] * size
        self.size = size
        self.index = 0
    
    def add(self, n):
        self.data[self.index] = n
        self.index = (self.index + 1) % self.size
    
    def to_list(self):
        return self.data[self.index:] + self.data[:self.index]

class TimeSeriesCollector(object):
    def __init__(self):
        self.hourly = {}
        self.hourly_timings = {}
        self.last_collection = time.time()
        self.stats = stats.fork()
    
    def collect(self):
        def get_or_add(key, new=0, d=self.hourly):
            value = d.get(key)
            if value is None:
                value = TimeSeries(60, new)
                d[key] = value
            return value
        
        for k, v in stats.get_gauge_stats().items():
            get_or_add("gauge:%s" % k).add(v)
        
        for k, v in self.stats.get_counter_stats(reset=True).items():
            get_or_add("counter:%s" % k).add(v)
        
        for k, v in self.stats.get_timing_stats(reset=True).items():
            data = map(v.histogram.get_percentile, [0.5, 0.999])
            get_or_add("timing:%s" % k, [0, 0], d=self.hourly_timings).add(data)
        
        self.last_collection = time.time()
    
    def start_twisted(self, collect_every=60):
        from twisted.internet import task
        self.task = task.LoopingCall(self.collect)
        self.task.start(collect_every)
        return self.task
    
    def stop_twisted(self):
        self.task.stop()
    
    def get(self, name):
        times = [self.last_collection + ((i - 59) * 60) for i in xrange(60)]
        if name in self.hourly:
            data = zip(times, self.hourly[name].to_list())
        else:
            data = [[a] + b for a, b in zip(times, self.hourly_timings[name].to_list())]
        return {name: data}
    
    def keys(self):
        return self.hourly.keys() + self.hourly_timings.keys()


#!/usr/bin/env python
from setuptools import setup, find_packages

trove_classifiers=[
    "Development Status :: 4 - Beta",
    "License :: OSI Approved :: BSD License",
    "License :: DFSG approved",
    "Intended Audience :: Developers",
    "Operating System :: OS Independent",
    "Natural Language :: English",
    "Programming Language :: Python",
    "Programming Language :: Python :: 2",
    "Programming Language :: Python :: 2.5",
    "Programming Language :: Python :: 2.6",
    "Topic :: Utilities",
    "Topic :: Software Development :: Libraries",
    ]

setup(name="ostrich",
      version="0.2.3",
      description="Python port of the Scala Ostrich library",
      author="Wade Simmons",
      author_email="wade@wades.im",
      url="http://github.com/wadey/python-ostrich",
      py_modules=['ostrich'],
      packages = find_packages(),
      test_suite="ostrich.test",
      license = "Apache 2.0",
      keywords="ostrich",
      setup_requires=['setuptools_trial', 'setuptools_pyflakes'],
      install_requires=['decorator'],
      tests_require=['mock'],
      classifiers=trove_classifiers,
      zip_safe = True)



import sys
import time
import math
import unittest

from ostrich import stats, timing, histogram

class StatsTest(unittest.TestCase):
    def setUp(self):
        stats.clear_all()
    
    def test_counters(self):
        stats.incr("widgets", 1)
        stats.incr("wodgets", 12)
        stats.incr("wodgets")
        self.assertEquals({"widgets": 1, "wodgets": 13}, stats.get_counter_stats())
    
    def test_timings_empty(self):
        stats.add_timing("test", 0)
        test = stats.get_timing("test")
        self.assertEqual(timing.TimingStat(1, 0, 0), test.get(reset=True))
        # the timings list will be empty here:
        self.assertEqual(timing.TimingStat(0, 0, 0), test.get())
    
    def test_timings_basic(self):
        stats.add_timing("test", 1)
        stats.add_timing("test", 2)
        stats.add_timing("test", 3)
        test = stats.get_timing("test")
        self.assertEqual(timing.TimingStat(3, 3, 1, 2.0, 2.0, histogram.Histogram(1, 2, 3)), test.get())
    
    def test_timings_report(self):
        x = 0
        with stats.time("hundred"):
            for i in xrange(100):
                x += i
        timings = stats.get_timing_stats()
        self.assertEquals(["hundred"], timings.keys())
        self.assertEquals(1, timings["hundred"].count)
        self.assertEquals(timings["hundred"].average, timings["hundred"].min)
        self.assertEquals(timings["hundred"].average, timings["hundred"].max)
    
    def test_timings_average(self):
        stats.add_timing("test", 0)
        test = stats.get_timing("test")
        self.assertEquals(timing.TimingStat(1, 0, 0), test.get())
    
    def test_timings_negative(self):
        stats.add_timing("test", 1)
        stats.add_timing("test", -1)
        test = stats.get_timing("test")
        self.assertEquals(timing.TimingStat(1, 1, 1, 1.0, 0.0, histogram.Histogram(1)), test.get())
    
    def test_timing_boundarys(self):
        stats.add_timing("test", sys.maxint)
        stats.add_timing("test", 5)
        sum = 5.0 + sys.maxint
        avg = sum / 2.0
        sumsq = 5.0 * 5.0 + float(sys.maxint) * sys.maxint
        partial = sumsq - sum * avg
        test = stats.get_timing("test")
        self.assertEquals(timing.TimingStat(2, sys.maxint, 5, avg, partial, histogram.Histogram(5, sys.maxint)), test.get())
    
    def test_timing_with(self):
        with stats.time("test"):
            time.sleep(0.01)
        test = stats.get_timing("test")
        self.assertTrue(test.get().average >= 10)
    
    def test_timing_decorator(self):
        @stats.time("test")
        def _test_timing_decorator():
            time.sleep(0.01)
        _test_timing_decorator()
        test = stats.get_timing("test")
        self.assertTrue(test.get().average >= 10)
    
    def test_timing_reset(self):
        x = 0
        
        with stats.time("hundred"):
            for i in xrange(100): x += i
        self.assertEqual(1, stats.get_timing_stats(reset=False)["hundred"].count)
        with stats.time("hundred"):
            for i in xrange(100): x += i
        self.assertEqual(2, stats.get_timing_stats(reset=False)["hundred"].count)
        self.assertEqual(2, stats.get_timing_stats(reset=True)["hundred"].count)
        with stats.time("hundred"):
            for i in xrange(100): x += i
        self.assertEqual(1, stats.get_timing_stats(reset=False)["hundred"].count)
    
    def test_timing_bundle(self):
        timing_stat = timing.TimingStat(3, 20, 10, 15.0, 50.0, histogram.Histogram(10, 15, 20))
        stats.add_timing("test", timing_stat)
        stats.add_timing("test", 25)
        test = stats.get_timing_stats(reset=False)["test"]
        self.assertEqual(4, test.count)
        self.assertEqual(17, test.average)
        self.assertEqual(6, int(test.std_dev))
        
        stats.clear_all()
        
        timing_stat1 = timing.TimingStat(2, 25, 15, 20.0, 50.0, histogram.Histogram(15, 25))
        timing_stat2 = timing.TimingStat(2, 20, 10, 15.0, 50.0, histogram.Histogram(10, 20))
        stats.add_timing("test", timing_stat1)
        stats.add_timing("test", timing_stat2)
        test = stats.get_timing_stats(reset=False)["test"]
        self.assertEqual(4, test.count)
        self.assertEqual(17, test.average)
        self.assertEqual(6, int(test.std_dev))
    
    def test_timing_add(self):
        x = 0
        with stats.time("hundred"):
            for i in xrange(100): x += 1
        self.assertEquals(1, len(stats.get_timing_stats(reset=False)))
        
        stats.add_timing("foobar", timing.TimingStat(1, 0, 0))
        self.assertEquals(2, len(stats.get_timing_stats(reset=False)))
        self.assertEquals(1, stats.get_timing_stats(reset=True)["foobar"].count)
        stats.add_timing("foobar", timing.TimingStat(3, 0, 0))
        self.assertEquals(3, stats.get_timing_stats(reset=False)["foobar"].count)
    
    # TODO: not implemented
    # def test_timing_external(self):
    #     pass
    
    def test_timing_report_sorted(self):
        stats.add_timing("alpha", timing.TimingStat(1, 0, 0))
        string = str(stats.get_timing_stats(reset=False)["alpha"])
        self.assertEquals("(average=0, count=1, maximum=0, minimum=0, p25=0, p50=0, p75=0, p90=0, p99=0, p999=0, p9999=0, standard_deviation=0)", string)
    
    # TODO
    # def test_timing_json_contains_histogram_buckets(self):
    #    pass
    
    def test_gauge_report(self):
        @stats.gauge("pi")
        def _pi():
            return math.pi
        
        stats.make_gauge("e", lambda: math.e)
        self.assertEquals({"e": math.e, "pi": math.pi}, stats.get_gauge_stats())
    
    def test_gauge_update(self):
        potatoes = [100.0]
        @stats.gauge("stew")
        def _stew():
            potatoes[0] += 1.0
            return potatoes[0]
        self.assertEquals({"stew": 101.0}, stats.get_gauge_stats())
        self.assertEquals({"stew": 102.0}, stats.get_gauge_stats())
        self.assertEquals({"stew": 103.0}, stats.get_gauge_stats())

    def test_fork(self):
        collection = stats.fork()
        stats.incr("widgets", 5)
        self.assertEquals({"widgets": 5}, collection.get_counter_stats())
        self.assertEquals({"widgets": 5}, stats.get_counter_stats(reset=True))
        stats.incr("widgets", 5)
        self.assertEquals({"widgets": 10}, collection.get_counter_stats())
        self.assertEquals({"widgets": 5}, stats.get_counter_stats(reset=True))








import unittest
import time

import mock

from ostrich import stats
from ostrich.time_series_collector import TimeSeriesCollector

real_time = time.time

class TimeSeriesCollectorTest(unittest.TestCase):
    def setUp(self):
        stats.clear_all()
        self.collector = TimeSeriesCollector()
    
    @mock.patch("time.time")
    def test_report_basic_stats(self, mock_time):
        my_time = real_time()
        mock_time.return_value = my_time
        
        stats.incr("cats")
        stats.incr("dogs", 3)
        self.collector.collect()
        my_time += 60
        mock_time.return_value = my_time
        stats.incr("dogs")
        self.collector.collect()
        
        data = self.collector.get("counter:dogs")
        self.assertEquals((my_time - (2 * 60), 0), data["counter:dogs"][57])
        self.assertEquals((my_time - 60, 3), data["counter:dogs"][58])
        self.assertEquals((my_time, 1), data["counter:dogs"][59])
    
